/*
GLDM - Graphical Liberty Display Manager

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef BASEINTERFACE_H_INCLUDED
#define BASEINTERFACE_H_INCLUDED

#include <string>

class AuthProvider;

/** A base class for all user interfaces. If you want to write a new
  * user interface, it needs to be derived from this class, and implement
  * all necessary functions. If you don't want to write your own window
  * management functions, you may want to consider using something like
  * SDLInterface.
  *
  * This just specifies an abstract class.
  */
class BaseInterface
{
    public:
    BaseInterface();
    ~BaseInterface();

    /** Returns the string representation of the user interface's class name.
      *
      * \return STL string containing the human readable name of the
      *         interface. For example, "FloatingCubeInterface"
      */
    virtual const std::string& getInterfaceName() const = 0;

    /** Activates the user interface and makes it fully visible to the user.
      */
    virtual void showSelf() = 0;

    /** Deactivates the user interface and hides it from the user. It is
      * called, for example, when the user successfully logs in, or when
      * s/he chooses to cancel the login attempt.
      */
    virtual void hideSelf() = 0;

    /** Paints a single frame in the animation of the UI. Please avoid
      * changing state of non-local variables in paintSelf() (including
      * statics) and do updating in updateSelf(), but as long as it's
      * practical.
      */
    virtual void paintSelf() = 0;

    /** Tells the UI to update itself, since ''k'' seconds has passed since
      * the last update.
      * \param k a decimal number specifying how much time has passed since
      *          the last update
      */
    virtual void updateSelf(double k) = 0;

    /** Tells the UI manager that we're finished with login, and it should
      * determine how to proceed based on other factors such as whether
      * the login was successful or whether the login was canceled.
      * \return A boolean specifying whether we're finished or not
      */
    virtual bool isFinished() = 0;

    /** When we're finished, UI manager can determine whether we finished
      * because the user canceled the login attempt.
      * \return A boolean specifying whether the user chose Cancel or not
      */
    virtual bool isCanceled() = 0;

    /** In case the interface is using an UI library which has its own event
      * handling mechanisms, which might allow the interface to use much
      * less CPU usage, it should implement its event waiting and collection
      * in this function. For example, if your SDL-based interface doesn't
      * really have much painting to do, you shouldn't throw your event
      * handling into paintSelf() or updateSelf() -- you should instead
      * put it into this dedicated function. This separation is done so we
      * can be more future-proof; the UI manager could be changed in a way
      * that might break the code if we didn't have a specialized event
      * collector.
      *
      * This function does not HAVE to be implemented.
      */
    virtual void grabEvents() {}

    /** Returns the user name chosen in the UI.
      * \return STL string specifying the UNIX login of selected user
      */
    virtual const std::string& getLogin() const = 0;

    /** Returns the password chosen in the UI.
      * \return STL string specifying the password of selected user
      */
    virtual const std::string& getPassword() const = 0;

    /** Returns the name of session chosen in the UI. This can be
      * something like 'gnome-session'.
      * \return STL string specifying selected session, or empty
      *         STL string meaning that we should pick a default.
      */
    virtual const std::string& getSessionName() const { static const std::string empty = ""; return empty; }

    /** Returns the pointer to auth provider which as reported success
      * in authorizing login data provided by the user. This will be
      * used by UI manager to verify the data once again, and also to use
      * authprovider's user-switching functionality via
      * AuthProvider::switchUser().
      */
    virtual AuthProvider* getSucceedingAuthProvider() const = 0;

};

#endif
